
# some defines from microcontroller firmware
# define COMMUNICATION_PACKET_SOF        0x01          // start of frame value
PACKET_SOF = 0x01
# define COMMUNICATION_PACKET_ADDR       0x30          // address (...yes, only this address is listened to)
PACKET_ADDR = 0x30
# define COMMUNICATION_PACKET_EOF        0x00          // end of frame value
PACKET_EOF = 0x00


def parse_num(_val):
    assert isinstance(_val, basestring)
    if _val.startswith('0x'):
        return int(_val, 16)
    else:
        return int(_val, 10)


def nibble_to_ascii(_val):
    if _val < 10:
        r = ord('0') + _val
    else:
        r = ord('a') + (_val - 10)
    return r


def uint8_to_nibbles(_byte):
    nibble_0 = nibble_to_ascii(_byte & 0x0f)
    nibble_1 = nibble_to_ascii((_byte >> 4) & 0x0f)
    return nibble_0, nibble_1


def get_checksum(_rawdata):
    checksum = 0
    for k in range(1, len(_rawdata)):
        checksum = checksum - _rawdata[k]
        checksum = checksum % 256
    return checksum


def packet_create_rawdata(_command_code, _payload_bytes):
    # builds the array of bytes to be sent
    rawdata = bytearray()
    # start of frame and command code
    rawdata.append(PACKET_SOF)
    rawdata.append(PACKET_ADDR)
    rawdata.append(_command_code)
    # append encoded payload data
    for val in _payload_bytes:
        nibble_0, nibble_1 = uint8_to_nibbles(val)
        rawdata.append(nibble_1)
        rawdata.append(nibble_0)
    # compute the checksum
    checksum = get_checksum(rawdata)
    rawdata.append(checksum)
    # if checksum does NOT equal PACKET_EOF, then add PACKET_EOF
    if checksum != PACKET_EOF:
        rawdata.append(PACKET_EOF)
    return rawdata


def packet_extract_data(_rawdata):
    # print _rawdata
    assert _rawdata[0] == PACKET_SOF
    assert _rawdata[1] == PACKET_ADDR
    command_code = _rawdata[2]
    # decode payload
    payload_bytes = list()
    for idx in range(3, len(_rawdata), 2):
        if _rawdata[idx] == PACKET_EOF or _rawdata[idx+1] == PACKET_EOF:
            break
        val = int(str(chr(_rawdata[idx])) + str(chr(_rawdata[idx+1])), 16)
        payload_bytes.append(val)
    # check checksum
    checksum = get_checksum(_rawdata[:-1])
    assert _rawdata[idx + 1] == checksum, 'Wrong checksum'
    if checksum == PACKET_EOF:
        assert (idx + 2) == len(_rawdata)
    else:
        assert _rawdata[idx + 2] == PACKET_EOF
        assert (idx + 2) == len(_rawdata)
    return command_code, payload_bytes

